package cn.ibizlab.ehr.util.aspect;

import lombok.extern.slf4j.Slf4j;
import cn.ibizlab.ehr.util.annotation.DEField;
import cn.ibizlab.ehr.util.enums.DupCheck;
import cn.ibizlab.ehr.util.errors.BadRequestAlertException;
import cn.ibizlab.ehr.util.filter.QueryFilter;
import cn.ibizlab.ehr.util.filter.SearchContextBase;
import cn.ibizlab.ehr.util.helper.DEFieldCacheMap;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.data.domain.Page;
import org.springframework.expression.EvaluationContext;
import org.springframework.expression.Expression;
import org.springframework.expression.ExpressionParser;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import java.util.Map;

/**
 * 属性重复值检查切面
 */
@Aspect
@Component
@Slf4j
public class DupCheckAspect {

    private final ExpressionParser parser = new SpelExpressionParser();

     /**
     * 实体[AttEndenceSetup]
     *
     * @param point
     */
    @AfterReturning(value = "(execution(* cn.ibizlab.ehr.core.*.service.*AttEndenceSetup*.create*(..))||execution(* cn.ibizlab.ehr.core.*.service.*AttEndenceSetup*.update*(..))||execution(* cn.ibizlab.ehr.core.*.service.*AttEndenceSetup*.save*(..))  ) && !execution(* cn.ibizlab.ehr.core.es.service.*.create*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.update*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.save*(..)) ")
    public void checkAttendencesetup(JoinPoint point) {
        check(point, "searchDefault");
    }
     /**
     * 实体[CodeItem]
     *
     * @param point
     */
    @AfterReturning(value = "(execution(* cn.ibizlab.ehr.core.*.service.*CodeItem*.create*(..))||execution(* cn.ibizlab.ehr.core.*.service.*CodeItem*.update*(..))||execution(* cn.ibizlab.ehr.core.*.service.*CodeItem*.save*(..))  ) && !execution(* cn.ibizlab.ehr.core.es.service.*.create*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.update*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.save*(..)) ")
    public void checkCodeitem(JoinPoint point) {
        check(point, "searchDefault");
    }
     /**
     * 实体[PcmProfileNumBeprefix]
     *
     * @param point
     */
    @AfterReturning(value = "(execution(* cn.ibizlab.ehr.core.*.service.*PcmProfileNumBeprefix*.create*(..))||execution(* cn.ibizlab.ehr.core.*.service.*PcmProfileNumBeprefix*.update*(..))||execution(* cn.ibizlab.ehr.core.*.service.*PcmProfileNumBeprefix*.save*(..))  ) && !execution(* cn.ibizlab.ehr.core.es.service.*.create*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.update*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.save*(..)) ")
    public void checkPcmprofilenumbeprefix(JoinPoint point) {
        check(point, "searchDefault");
    }
     /**
     * 实体[PcmProfileSpyy]
     *
     * @param point
     */
    @AfterReturning(value = "(execution(* cn.ibizlab.ehr.core.*.service.*PcmProfileSpyy*.create*(..))||execution(* cn.ibizlab.ehr.core.*.service.*PcmProfileSpyy*.update*(..))||execution(* cn.ibizlab.ehr.core.*.service.*PcmProfileSpyy*.save*(..))  ) && !execution(* cn.ibizlab.ehr.core.es.service.*.create*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.update*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.save*(..)) ")
    public void checkPcmprofilespyy(JoinPoint point) {
        check(point, "searchDefault");
    }
     /**
     * 实体[PcmZpmegl]
     *
     * @param point
     */
    @AfterReturning(value = "(execution(* cn.ibizlab.ehr.core.*.service.*PcmZpmegl*.create*(..))||execution(* cn.ibizlab.ehr.core.*.service.*PcmZpmegl*.update*(..))||execution(* cn.ibizlab.ehr.core.*.service.*PcmZpmegl*.save*(..))  ) && !execution(* cn.ibizlab.ehr.core.es.service.*.create*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.update*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.save*(..)) ")
    public void checkPcmzpmegl(JoinPoint point) {
        check(point, "searchDefault");
    }
     /**
     * 实体[PimArchives]
     *
     * @param point
     */
    @AfterReturning(value = "(execution(* cn.ibizlab.ehr.core.*.service.*PimArchives*.create*(..))||execution(* cn.ibizlab.ehr.core.*.service.*PimArchives*.update*(..))||execution(* cn.ibizlab.ehr.core.*.service.*PimArchives*.save*(..))  ) && !execution(* cn.ibizlab.ehr.core.es.service.*.create*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.update*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.save*(..)) ")
    public void checkPimarchives(JoinPoint point) {
        check(point, "searchDefault");
    }
     /**
     * 实体[PimPerson]
     *
     * @param point
     */
    @AfterReturning(value = "(execution(* cn.ibizlab.ehr.core.*.service.*PimPerson*.create*(..))||execution(* cn.ibizlab.ehr.core.*.service.*PimPerson*.update*(..))||execution(* cn.ibizlab.ehr.core.*.service.*PimPerson*.save*(..))  ) && !execution(* cn.ibizlab.ehr.core.es.service.*.create*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.update*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.save*(..)) ")
    public void checkPimperson(JoinPoint point) {
        check(point, "searchDefault");
    }
     /**
     * 实体[SalItem]
     *
     * @param point
     */
    @AfterReturning(value = "(execution(* cn.ibizlab.ehr.core.*.service.*SalItem*.create*(..))||execution(* cn.ibizlab.ehr.core.*.service.*SalItem*.update*(..))||execution(* cn.ibizlab.ehr.core.*.service.*SalItem*.save*(..))  ) && !execution(* cn.ibizlab.ehr.core.es.service.*.create*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.update*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.save*(..)) ")
    public void checkSalitem(JoinPoint point) {
        check(point, "searchDefault");
    }
     /**
     * 实体[SalSchemeItem]
     *
     * @param point
     */
    @AfterReturning(value = "(execution(* cn.ibizlab.ehr.core.*.service.*SalSchemeItem*.create*(..))||execution(* cn.ibizlab.ehr.core.*.service.*SalSchemeItem*.update*(..))||execution(* cn.ibizlab.ehr.core.*.service.*SalSchemeItem*.save*(..))  ) && !execution(* cn.ibizlab.ehr.core.es.service.*.create*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.update*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.save*(..)) ")
    public void checkSalschemeitem(JoinPoint point) {
        check(point, "searchDefault");
    }
     /**
     * 实体[SocRuleDetail]
     *
     * @param point
     */
    @AfterReturning(value = "(execution(* cn.ibizlab.ehr.core.*.service.*SocRuleDetail*.create*(..))||execution(* cn.ibizlab.ehr.core.*.service.*SocRuleDetail*.update*(..))||execution(* cn.ibizlab.ehr.core.*.service.*SocRuleDetail*.save*(..))  ) && !execution(* cn.ibizlab.ehr.core.es.service.*.create*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.update*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.save*(..)) ")
    public void checkSocruledetail(JoinPoint point) {
        check(point, "searchDefault");
    }
     /**
     * 实体[VacHolidayRules]
     *
     * @param point
     */
    @AfterReturning(value = "(execution(* cn.ibizlab.ehr.core.*.service.*VacHolidayRules*.create*(..))||execution(* cn.ibizlab.ehr.core.*.service.*VacHolidayRules*.update*(..))||execution(* cn.ibizlab.ehr.core.*.service.*VacHolidayRules*.save*(..))  ) && !execution(* cn.ibizlab.ehr.core.es.service.*.create*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.update*(..)) && !execution(* cn.ibizlab.ehr.core.es.service.*.save*(..)) ")
    public void checkVacholidayrules(JoinPoint point) {
        check(point, "searchDefault");
    }

    /**
     * 实体属性重复值检查
     * @param point 切点
     * @param defaultDS 实体默认数据集名称
     */
    private void check(JoinPoint point, String defaultDS) {
        Object[] args = point.getArgs();
        if (args.length > 0) {
            Object entity = args[0];
            Object service = point.getTarget();
            Map<String, DEField> deFields = DEFieldCacheMap.getDEFields(entity.getClass());
            for (Map.Entry<String, DEField> deField : deFields.entrySet()) {
                String fieldName = deField.getKey();
                DEField fieldAnnotation = deField.getValue();
                DupCheck dupCheck = fieldAnnotation.dupCheck();
                String dupCheckField=fieldAnnotation.dupCheckField();
                if (dupCheck == DupCheck.ALL) {
                    Object newValue =getDEFieldValue(entity,fieldName);
                    //获取searchContext
                    EvaluationContext searchContextCtx = new StandardEvaluationContext();
                    searchContextCtx.setVariable("service", service);
                    Expression searchContextExp = parser.parseExpression("#service.getSearchContext()");
                    SearchContextBase searchContext = searchContextExp.getValue(searchContextCtx, SearchContextBase.class);

                    //设置检查属性值
                    QueryFilter filter = new QueryFilter();
                    setValue(entity, filter, fieldName, newValue);

                    //设定重复值检查范围
                    if(!StringUtils.isEmpty(dupCheckField)) {
                        Object dupFieldValue=getDEFieldValue(entity,dupCheckField);
                        setValue(entity, filter, dupCheckField, dupFieldValue);
                    }
                    searchContext.setFilter(filter);
                    //使用当前值到数据库中进行查询，判断是否重复
                    EvaluationContext oldValueMappingCtx = new StandardEvaluationContext();
                    oldValueMappingCtx.setVariable("service", service);
                    oldValueMappingCtx.setVariable("searchContext", searchContext);
                    Expression oldValueMappingExp = parser.parseExpression(String.format("#service.%s(#searchContext)", defaultDS));
                    Page oldData = oldValueMappingExp.getValue(oldValueMappingCtx, Page.class);
                    if (!ObjectUtils.isEmpty(oldData) && !ObjectUtils.isEmpty(oldData.getContent()) && oldData.getContent().size() > 1) {
                        throw new BadRequestAlertException(String.format("数据保存失败，属性[%s]:值[%s]已存在!", fieldName, newValue), "DupCheckAspect", "DupCheck");
                    }
                }
            }
        }
    }

    /**
     * 获取实体属性值
     * @param entity
     * @param fieldName
     * @return
     */
    private Object getDEFieldValue(Object entity, String fieldName) {
        EvaluationContext exMappingCtx = new StandardEvaluationContext();
        exMappingCtx.setVariable("entity", entity);
        Expression esMappingExp = parser.parseExpression(String.format("#entity.get(\"%s\")", fieldName));
        return esMappingExp.getValue(exMappingCtx);
    }

    /**
     * 设置filter
     * @param entity
     * @param filter
     * @param value
     */
    private void setValue(Object entity , QueryFilter filter, String fieldName, Object value){
        if(ObjectUtils.isEmpty(value)) {
            filter.isnull(DEFieldCacheMap.getFieldColumnName(entity.getClass(), fieldName));
        }
        else {
            filter.eq(DEFieldCacheMap.getFieldColumnName(entity.getClass(), fieldName), value);
        }
    }
}

